// rng.h
//
// Created 1998 by Jouni Seppnen.
//
// @(#)$Id: rng.h,v 1.2 1998/08/31 20:01:12 jks Exp $
//

#ifndef RNG_H
#define RNG_H

#include <stdlib.h>
#include <time.h>

// Actually, an abstract class randomNumberGenerator would be nice.
// However, random numbers are generated all the time in Bassist,
// and there's typically just one generator, so doing a table lookup
// for a virtual function just to generate one random number seems
// a waste.

/**
      This class is an interface to the random number generator 
      in the C Library. Therefore, the quality of the random numbers
      generated depends on your C Library; modern GNU/Linux systems 
      seem to have a pretty good generator.

      @author Jouni Seppnen
 */
      
class libcRandomNumberGenerator {

public:

    /**
         Constructor. Seeds the RNG with the given seed.
	 
	 @param seed The seed for the random number generator.
     */
    libcRandomNumberGenerator(long seed) {setSeed(seed);};
    /**
         Constructor. Seeds the RNG with the time.
     */
    libcRandomNumberGenerator() {setSeed(time(NULL));};
    
    /**
         Sets the seed of the RNG. The sequence generated by
	 the RNG is repeatable by calling this function again
	 with the same seed.

         @memo Seed the RNG

	 @param seed The seed for the random number generator.
     */
    void setSeed(long seed) {srand((unsigned int)seed);};
    /**
         Returns a random number uniformly distributed in
	 the specified interval.

         @memo Random number uniformly distributed in [a,b]

	 @param a Lower limit of the interval.
	 @param b Upper limit of the interval.
	 @return A random number.
     */
    double random(double a, double b) {
	return a + (b-a) * 1.0*rand()/RAND_MAX;
    }
};

#endif // RNG_H

//------------------------------------------------
// Local Variables:				//
// mode: c++					//
// make-backup-files: t				//
// c-basic-offset: 4                            //
// End:						//
//------------------------------------------------
