// scanner.h
//
// Created 1997 by Karri-Pekka Laakso
//
// @(#)$Id: scanner.h,v 1.5 1999/05/07 08:35:33 jks Exp $
//

#ifndef SCANNER_H
#define SCANNER_H

#include <stdio.h>

// Forward declaration
class parameter;

/**
  	Class scanner offers an interface to read a given file according
  	to a wanted format. This class handles all reading of datafiles.
  	For each datafile there exists one instance of scanner.
  	@author Karri-Pekka Laakso
*/

class scanner {
public:
    /// Maximum length of the identifier
    enum { MAXID = 128 };

protected:
    /// Protected copy constructor (prevents accidental copying)
    scanner(scanner &) { }

    /// Maximum size of the buffer
    enum { BUFSIZE = 1024 };

    /// Reading format for buffer
    static char bufReadFormat[20];

    /// Reading format for identifiers
    static char idReadFormat[20];

    /// Buffer
    char buffer[BUFSIZE+1];

    /// Buffer for storing just read identifier
    char s[MAXID+2];

    /// Format for buffer
    char bufFillFormat[20];

    /// Name of the target file
    char *filename;
    
    /// Target file
    FILE *file;

    /// Indicates the place where we are currently in the buffer 
    int ind;

    /** This attribute has a value true, if current line is longer
	than BUFSIZE and false otherwise.
    */
    bool longLine;

    /// Number of the line that we are currently scanning
    int currentLine;

    /** Attribute that has a value true if end of file has been reached,
	otherwise false.
      */
    bool eofReached;

    /**
       Eats all the whitespaces from the input buffer and reads
       a new line, if end of buffer was reached.
       @return False if end of file was reached during eating
       whitespaces, otherwise true.
      */
    bool fillBuffer();

public:
    /**
       Constructor
       @param Name of the target file
      */
    scanner(char *fname);

    /**
       Requests the argument string from the input line.
       If it turns out that requested string is not found not,
       method exists the program.
       @param *string searched string from the input line
    */
    void get(char *string);

    /**
	Checks if the argument string is in the input.
    	If the argument string was found, moves the input
    	forward, if it was not found, no move is done.
    	@param *string argument string
    	@return True if argument string was found, otherwise false.
    */
    bool ask(char *string);
    
   /**
   	Copies the next identifier from the input stream.
	An indentifier may consist of chars a-z, A-Z, 0-9 and
	_. 
   	@param 
   	@return True, if an identifier was found, otherwise false.
   */
   bool  getId(char *string);

    /**
	Tries to read a double from the input stream and returns
	its value. Moves the input stream forward. If read fails,
	method exists.
    	@return A read double value from the input stream.
    */
    double getDoubleValue();
//     bool askObservationStatus(int *p, char *name);

    /**
    	Returns a pointer to the next identifier in the input stream.
    	Moves the input forward. Subsequent calls will overwrite
    	the contents of the returned pointer. 
    	@param moveForward moves reading pointer forward
    	if this parameter has a value true
    	@return  A pointer to the next identifier in the input stream.
    */
    char *next(bool moveForward = true);

    /**
       Returns value of the attribute eofReached.
       @return True if end of file is reached, otherwise false.
      */
    inline bool eof() { return eofReached; }

    /**
       Returns number of the line that we are currently scanning 
       @return Value of the attribute currentLine
    */
    inline int line() { return currentLine; }

    /**
       Returns the name of the target file
       @return The name of the target file
      */
    inline char *fileName() { return filename; }

    /// Destructor
    ~scanner();
};

#endif // SCANNER_H

//------------------------------------------------
// Local Variables:				//
// mode: c++					//
// make-backup-files: t				//
// End:						//
//------------------------------------------------
